@@ -1,8 +1,8 @@
 # global variables for dop/uere/error functions (special axes)
 DOP.LIST <- list(unknown=list(axes=NA,geo=NA,DOP=NA,VAR=NA,COV=NA) ,
-                 horizontal=list(axes=c("x","y"),geo=c("longitude","latitude"),DOP="HDOP",VAR="VAR.xy",COV=c("COV.x.x","COV.x.y","COV.y.y")),
+                 horizontal=list(axes=c("x","y"),geo=c("longitude","latitude"),DOP="HDOP",VAR="VAR.xy",COV=c("COV.x.x","COV.x.y","COV.y.y"),COV.geo=c("COV.major","COV.minor","COV.angle")),
                  vertical=list(axes="z",geo="z",DOP="VDOP",VAR="VAR.z",COV=NA),
-                 speed=list(axes=c("vx","vy"),geo=c("speed","heading"),DOP="SDOP",VAR="VAR.v",COV=c("COV.vx.vx","COV.vx.vy","COV.vy.vy")) )
+                 speed=list(axes=c("vx","vy"),geo=c("speed","heading"),DOP="SDOP",VAR="VAR.v",COV=c("COV.vx.vx","COV.vx.vy","COV.vy.vy"),COV.geo=NA) )
 
 
 # is the data calibrated
@@ -453,7 +453,7 @@ summary.UERE.list <- function(object,level=0.95,drop=TRUE,CI=FALSE,...)
     if(CI)
     {
       # is N correct here for 1D and 2D ???
-      TAB[[i]] <- sapply(1:DIM[2],function(j){ chisq.ci(Zsq[i,j],COV=VAR.Zsq[i,j]/N[i,j],level=level) }) # (3CIS,models)
+      TAB[[i]] <- sapply(1:DIM[2],function(j){ chisq.ci(Zsq[i,j],VAR=VAR.Zsq[i,j]/N[i,j],level=level) }) # (3CIS,models)
       TAB[[i]] <- cbind(AIC[i,],t(TAB[[i]]))
       colnames(TAB[[i]]) <- c("\u0394AICc","(       ","Z[red]\u00B2","       )")
     }
@@ -567,7 +567,7 @@ get.error <- function(DATA,CTMM,flag=FALSE,circle=FALSE,DIM=FALSE)
 
       error <- (CTMM$error*DATA[[DOP]])^2/length(axes)
     }
-    else # fitted errors - no HDOP
+    else # fitted errors - no HDOP - assign HDOP=1
     {
       FLAG <- 1
       if(flag) { return(FLAG) }
